// java
package edu.washington.cac.myuw.servlet;

import java.util.Enumeration;
import java.util.Hashtable;

/** LastModifiedTimes manages a number of last modified times, each
    one identified by a string.

    The times are stored in a hashtable as Longs, with the strings as the
    keys.
    
    @author Fang Lin, Greg Barnes
    @version 2.0 
  */

public class LastModifiedTimes {
    public static final int INITIAL_CAPACITY = 19;
    private Hashtable lastModifiedTimes = new Hashtable(INITIAL_CAPACITY);
    private Hashtable expireTimes = new Hashtable(INITIAL_CAPACITY);

    public static final long ALREADY_EXPIRED = -1;
//    public static final long EXPIRE_AFTER = 900000; // millies

    public long getLastModified(String id) {
        Long l = (Long) lastModifiedTimes.get(id);
  
        if (l == null) {
  	  // first time for this id
  	    setLastModified(id);
  	    return ALREADY_EXPIRED;
        } else {
  	    long lastModifiedTime = l.longValue();
            Long expire = (Long) expireTimes.get(id);

  	    if (expire != null && 
	      (System.currentTimeMillis() - lastModifiedTime) > 
		expire.longValue())
	    {
  	        setLastModified(id); // reset it
  	        return ALREADY_EXPIRED;
  	    } else {
  	        return lastModifiedTime;
  	    }
        }
    }
  
      /** set the last modified time for the given id
	@param id ID of the last modified time
	*/
    public void setLastModified(String id) {
        long lastmt = System.currentTimeMillis()/1000*1000;
        lastModifiedTimes.put(id, new Long(lastmt));
    }


      /** set the last modified time for the given id with an expiration
	@param id ID of the last modified time
	@param expire milliseconds until we assume the page has expired
	*/
    public void setLastModified(String id, long expire)
    {
	setLastModified(id);
        expireTimes.put(id, new Long(expire));
    }
  
      /** set the last modified time for all id's
        */
    public void setLastModified() {
        Long lastmt = new Long(System.currentTimeMillis()/1000*1000);
	
        for (Enumeration e = lastModifiedTimes.keys(); e.hasMoreElements() ; ) {
  	    lastModifiedTimes.put(e.nextElement(), lastmt);
        }
    }

      /** set the last modified time for all id's, along with an expiration
	  for all id's that <em>already have expirations</em>
	  @param expire milliseconds until we assume the page has expired
        */
    public void setLastModified(long expire) {
	setLastModified();
	
        for (Enumeration e = expireTimes.keys(); e.hasMoreElements() ; ) {
  	    expireTimes.put(e.nextElement(), new Long(expire));
        }
    }
}
