// java

package edu.washington.cac.myuw.servlet.session;

/** Interface that indicates an item is cacheable in a <code>Profile</code>
  @author Greg Barnes
  @version 1.3
 */
public interface Cacheable {
  /** set important information so we can load the object 
    @param profile The cache where the object will be stored
   */
  public void setProfile(Profile profile);

  /** get the object from persistant storage 
    @return The object, loaded
   */
  public Cacheable load();    

  /** @return did we fail to get the object? */
  public boolean loadFailed();

  /** write the object to persistant storage
    @exception java.sql.SQLException If the write fails
   */
  public void save() throws java.sql.SQLException;            

  /** @return Whether the object expired and needs to be reloaded */
  public boolean expired();

  /** @return The key used to find this value in the cache */
  public String getCacheKey();
}
