package edu.washington.cac.myuw.servlet.session;

import javax.servlet.http.HttpSession;

/**
  Stores an HttpSession and its id

  Ideally, we could always get a session's id from the session.  But
  lately some servlet containers have decided they need not give you
  an ID of an invalidated session, so we need to store the two together
  so we can get the ID if the session is invalidated.

  @author Greg Barnes
  @version 1.0
 */
class HttpSessionAndId
{
  /** The session */
  private HttpSession session;

  /** The id */
  private String id;

  /**
    Create a new instance

    @param session The session
    @param id The id
   */
  HttpSessionAndId(HttpSession session)
  {
    this.session = session;
    this.id = (session == null ? null : session.getId());
  }

  /**
    Get the id
    @return the id
   */
  String getId()
  {
    return this.id;
  }

  /**
    Get the session
    @return the session
   */
  HttpSession getSession()
  {
    return this.session;
  }
}
