// java

package edu.washington.cac.myuw.servlet.test;

import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.Locale;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;

/**
  Class to implement the HttpServletResponse class using a 
  TestServletOutputStream,
  for use in testing classes that use ServletOutputStreams.

  @author Greg Barnes
  @version 1.0
 */
public class TestHttpServletResponse implements HttpServletResponse 
{
  private TestServletOutputStream tsos;
  private PrintWriter pw;

  /** Create a new object that prints to a particular PrintStream
    */
  public TestHttpServletResponse(PrintStream ps) 
  {
    this.pw = new PrintWriter(ps);
    this.tsos = new TestServletOutputStream(ps);
  }

  public void addCookie(Cookie cookie) 
  {
    pw.println("addCookie: " + cookie);
  }

  public void addHeader(String name, String value)
  {
    pw.println("addHeader: " + name + " = " + value);
  }

  public void addDateHeader(String name, long date)
  {
    pw.println("addDateHeader: " + name + " = " + date);
  }

  public void addIntHeader(String name, int value)
  {
    pw.println("addIntHeader: " + name + " = " + value);
  }

  public boolean containsHeader(String name)
  {
    return false;
  }

  public void setStatus(int sc) 
  {
    setStatus(sc, "");
  }

  // deprecated
  public void setStatus(int sc, String sm) 
  {
    pw.println("setStatus: " + sc + ", " + sm);
  }

  public void setHeader(String name, String value) 
  {
    pw.println("setHeader: " + name + "=" + value);
  }

  public void setIntHeader(String name, int value)
  {
    pw.println("setIntHeader: " + name + "=" + value);
  }

  public void setDateHeader(String name, long date)
  {
    pw.println("setDateHeader: " + name + "=" + date);
  }

  public void sendError(int sc, String msg) throws IOException
  {
    pw.println("sendError: " + sc + ";" + msg);
  }

  public void sendError(int sc) throws IOException
  {
    sendError(sc, "");
  }

  public void sendRedirect(String location) throws IOException
  {
    pw.println("sendRedirect: " + location);
  }

  public String encodeUrl(String url)
  {
    return url;
  }

  public String encodeRedirectUrl(String url)
  {
    return url;
  }

  /* ServletResponse interface methods */

  public void setContentLength(int len)
  {
    pw.println("setContentLength: " + len);
  }

  public void setContentType(String type)
  {
    pw.println("setContentType: " + type);
  }

  public String getContentType()
  {
    pw.println("getContentType");
    return "";
  }

  public ServletOutputStream getOutputStream() throws IOException
  {
    return tsos;
  }

  public PrintWriter getWriter() throws IOException
  {
    return pw;
  }

  public void setCharacterEncoding(String charset)
  {
    pw.println("setCharacterEncoding: " + charset);
  }

  public String getCharacterEncoding()
  {
    pw.println("getCharacterEncoding");
    return null;
  }

  public void setLocale(Locale loc)
  {
    pw.println("setLocale: " + loc);
  }
 
  public Locale getLocale()
  {
    pw.println("getLocale");
    return null;
  }

  public void setBufferSize(int size)
  {
    pw.println("setBufferSize");
  }

  public int getBufferSize()
  {
    pw.println("getBufferSize");
    return 0;
  }

  public void flushBuffer() throws java.io.IOException
  {
    pw.println("flushBuffer");
  }

  public boolean isCommitted()
  {
    pw.println("isCommitted");
    return false;
  }

  public void resetBuffer()
  {
    pw.println("resetbuffer");
  }
 
  public void reset()
  {
    pw.println("reset");
  }
 
  public String encodeURL(String url)
  {
    return url;
  }

  public String encodeRedirectURL(String url)
  {
    return url;
  }
}
