package pubevents.addevents;

import javax.servlet.http.HttpServletRequest;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.text.SimpleDateFormat;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.Recurrence;
import edu.washington.cac.calendar.data.NoRecurrence;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.calendar.data.Event;
import edu.washington.cac.calendar.data.Location;
import edu.washington.cac.calendar.data.Sponsor;
import edu.washington.cac.calendar.data.Keyword;

import edu.washington.cac.calendar.db.Caldata;
import edu.washington.cac.calendar.db.Fields;
import edu.washington.cac.calendar.db.PubEventsQueries;
import edu.washington.cac.calendar.db.Tables;
import edu.washington.cac.myuw.servlet.ParameterException;

import pubevents.PEParameter;
import pubevents.util.ConvertRoutines;

/** SqlStatements: sql statments to add/update/delete events in db tables.

  @author Leman Chung, Greg Barnes
  @version 2.2
 */
public class SqlStatements extends EventData 
{
  String temp_string;

  /** And in a select statement */
  private static final String AND = " and ";

  /** Start of the statement to select sponsors */
  private static final String SELECT_SPONSORS_START =
      "select " + Fields.SSPONSOR_ID.getSelectName() + " from "
      + Tables.SPONSORS.getName() + " where "
      + Fields.SPUBLIC.getStmtName() + "=\'T\'" + AND 
      + Fields.SNAME.getStmtName() + "=?" + AND
      + Fields.SPHONE.getStmtName();

  /** Part of a select statement to match a non-null field */
  private static final String EQUALS_VAL = "=?";

  /** Part of a select statement to match a null field */
  private static final String IS_NULL = " is null ";

  /** @return the sponsors that match the fields in a form
     @param req HTTP request object containing the form values
     @exception SQLException If there's a database access problem
   */
  public ResultSet selectSponsors(HttpServletRequest req) throws SQLException
  {
    PEParameter pep = new PEParameter(req);

    String sql = SELECT_SPONSORS_START +
                 ((pep.getPhone() == null) ? IS_NULL : EQUALS_VAL) +
                 AND + Fields.SEMAIL.getStmtName() +
                 ((pep.getEmail() == null) ? IS_NULL : EQUALS_VAL) +
                 AND + Fields.SLINK.getStmtName() +
                 ((pep.getSlink() == null) ? IS_NULL : EQUALS_VAL);

//System.err.println(sql);
    PreparedStatement s = getPreparedStatement(sql);
    s.setString(1, Fields.SNAME.sizeToFit(pep.getSponsorName()));

    // Keep track of which '?' we're filling in in the PreparedStatement
    int currentField = 2;

    if (pep.getPhone() != null) {
      s.setString(currentField, Fields.SPHONE.sizeToFit(pep.getPhone()));
      currentField++;
    }

    if (pep.getEmail() != null) {
      s.setString(currentField, Fields.SEMAIL.sizeToFit(pep.getEmail()));
      currentField++;
    }

    if (pep.getSlink() != null) {
      s.setString(currentField, 
          Fields.SLINK.sizeToFit(ConvertRoutines.insertHttp(pep.getSlink())));
    }

    return s.executeQuery();
  }

  /** SQL to check that an event exists with a particular sponsor */
  private static final String SELECT_SPONSOR_FROM_EVENTS = 
      "select " + Fields.ESPONSOR_ID.getSelectName() + " from "
      + Tables.EVENTS.getName() + " where "
      + Fields.ESPONSOR_ID.getStmtName() + "=?";

  public ResultSet selectSponsorFromEvents(int sponsorid)
    throws SQLException
  {
      PreparedStatement s = null;
      ResultSet rs = null;

      s = getPreparedStatement(SELECT_SPONSOR_FROM_EVENTS);
      s.setInt(1, sponsorid);
      rs = s.executeQuery();
      return rs;
  }

  /** add a sponsor via edu.washington.cac.calendar.data.Sponsor.java */
  public int addSponsors(HttpServletRequest req, String uid)
      throws SQLException, CaldataException
  {
    PEParameter pep = new PEParameter(req);
    // set public to true over here.   pep.set
    User user = new User(uid);
    Sponsor sp = new Sponsor(pep.getSponsorID(), pep.getSponsorName(),
        pep.getPhone(), pep.getEmail(), pep.getSlink(),
        user, pep.getPublic());

    try {
        sp.add(user);
    } finally {
        // close();
    }
    return sp.getId();
  }

  /** Start of a query for a location */
  private static final String SELECT_LOCATIONS_START = 
      "select " + Fields.LLOCATION_ID.getSelectName() + " from "
      + Tables.LOCATIONS.getName() + " where " 
      + Fields.LPUBLIC.getStmtName() + "=\'T\'" + AND
      + Fields.LADDRESS.getStmtName() + "= ?" + AND
      + Fields.LLINK.getStmtName();

  /** @return A location matching the fields in a request form, if one
     exists
      @param req HTTP request object containing the form values
      @exception SQLException If there's a problem
   */
  public ResultSet selectLocations(HttpServletRequest req) throws SQLException
  {
    PEParameter pep = new PEParameter(req);

    String sql = SELECT_LOCATIONS_START +
                 ((pep.getLlink() == null) ? IS_NULL : EQUALS_VAL);

    PreparedStatement s = getPreparedStatement(sql);
    s.setString(1, Fields.LADDRESS.sizeToFit(pep.getAddress()));

    if (pep.getLlink() != null) {
      s.setString(2, 
          Fields.LLINK.sizeToFit(ConvertRoutines.insertHttp(pep.getLlink())));
    }

    return s.executeQuery();
  }

  /** SQL to check whether a location is still used by any event */
  private static final String SELECT_LOCATION_FROM_EVENTS =
      "select " + Fields.ELOCATION_ID.getSelectName() + " from "
      + Tables.EVENTS.getName() + " where "
      + Fields.ELOCATION_ID.getStmtName() + "=?";

  public ResultSet selectLocFromEvents(int locationid)
    throws SQLException
  {
      PreparedStatement s = null;
      ResultSet rs = null;

          s = getPreparedStatement(SELECT_LOCATION_FROM_EVENTS);
          s.setInt(1, locationid);
          rs = s.executeQuery();
      return rs;
  }


  /** add a location by using edu.washington.cac.calendar.data.Location.java */
  public int addLocations(HttpServletRequest req, String uid)
      throws SQLException, CaldataException
  {
    PEParameter pep = new PEParameter(req);
    User user = new User(uid);
    Location loc = new Location(pep.getLocationID(), pep.getAddress(),
        pep.getSubaddress(), pep.getLlink(),
        user, pep.getPublic());

    try {
        loc.add(user);
    } finally {
      // close();
    }
    return loc.getId();
  }

  /** select from Users table in order to get userid (integer). */
  private static final String SELECT_USERS =
      "select " + Fields.UUSER_ID.getSelectName() + " from "
      + Tables.USERS.getName() + " where "
      + Fields.UUSERNAME.getStmtName() + "=?";

  public ResultSet selectUsers(String username)
    throws SQLException
  {
      PreparedStatement s = null;
      ResultSet rs = null;
      if (username == null) {
          return rs;
      }
      s = getPreparedStatement(SELECT_USERS);
      s.setString(1, username);
      rs = s.executeQuery();
      return rs;
  }

  public int addUsers(String uid)
      throws SQLException, CaldataException
  {
    int userid;
    User user = new User(uid);
    Caldata caldata = new Caldata();

    try {
      userid = caldata.add(user, false);
                                          // false means not via login.
    } finally {
      // close();
    }
    return userid;
  }

  /** SQL to add an entry to the authorization table */
  private static final String ADD_AUTH = 
      "insert into " + Tables.AUTH + " (" + 
      Fields.AUSERID.getInsertName() + ", " + 
      Fields.AUSERTYPE.getInsertName() + ", " + 
      Fields.ALASTNAME.getInsertName() + ", " + 
      Fields.AFIRSTNAME.getInsertName() + ", " + 
      Fields.APHONE.getInsertName() + ", " + 
      Fields.AEMAIL.getInsertName() + ", " + 
      Fields.ADEPARTMENT.getInsertName() + ") values (?, ?, ?, ?, ?, ?, ?)";

  public void addAuth(HttpServletRequest req, int userid)
    throws SQLException
  {
  PEParameter pep = new PEParameter(req);
      PreparedStatement s = null;

      try {
          s = getPreparedStatement(ADD_AUTH);

          String lastname = pep.getLastname();
          String firstname = pep.getFirstname();
          String uphone = pep.getUphone();
          String uemail = pep.getUemail();
          String dept = pep.getDepartment();

          s.setInt(1, userid);
          s.setString(2, pep.getUsertype());
          s.setString(3, Fields.ALASTNAME.sizeToFit(lastname));
          s.setString(4, Fields.AFIRSTNAME.sizeToFit(firstname));
          s.setString(5, Fields.APHONE.sizeToFit(uphone));
          s.setString(6, Fields.AEMAIL.sizeToFit(uemail));
          s.setString(7, Fields.ADEPARTMENT.sizeToFit(dept));
          s.executeUpdate();
      } finally {
          close();
      }
  }

  /**
    @return An event with the given id
    @param eventid ID of the event
    @exception SQLException If there's a database access problem
   */
  public ResultSet selectEvents(int eventid) throws SQLException
  {
    return 
        executeQuery(new PubEventsQueries().selectAnEvent(eventid).toString());
  }

  public int addEvents(HttpServletRequest req, String uid, 
                        int locationid, int sponsorid)
      throws SQLException, CaldataException
  {
    PEParameter pep = new PEParameter(req);
    User user = new User(uid);

    Event ev = new Event (pep, user,
                          locationid, sponsorid);

    try {
      ev.add(user);
    } finally {
      // close();
    }
    return ev.getId();
  }

  /** SQL to add an alert */
  private static final String ADD_ALERT = 
      "insert into " + Tables.EVENTS + " (" +
      Fields.ESEQ.getInsertName() + ", " + 
      Fields.ELASTMOD.getInsertName() + ", " +
      Fields.EPUBLIC.getInsertName() + ", " + 
      Fields.ECREATED.getInsertName() + ", " +
      Fields.ESTARTDATE.getInsertName() + ", " +
      Fields.EENDDATE.getInsertName() + ", " +
      Fields.ESHORTDESC.getInsertName() + ", " +
      Fields.ELONGDESC.getInsertName() + ", " +
      Fields.ELONGDESC1.getInsertName() + ", " +
      Fields.ELINK.getInsertName() + ", " +
      Fields.ESTATUS.getInsertName() + ", " +
      Fields.ELOCATION_ID.getInsertName() + ", " +
      Fields.ESPONSOR_ID.getInsertName() + ", " +
      Fields.ECREATOR.getInsertName() + ", " +
      Fields.ERECURRING_STATUS.getInsertName() + 
      ") values (1, " + Caldata.now() + ", \'T\', " + Caldata.now() +
      ", ?, ?, ?, ?, ?, ?, \'F\', 1, 1, ?, '" + 
      Recurrence.NO_RECUR_VALUE + "')";

  /** Add an alert
    @param req HTTP request object with the alert data
    @param uid Person entering the alert
    @exception SQLException If there's a database access problem
    @exception ParameterException If there's a missing value in the form
    @exception CaldataException If there's a problem with the form data
   */

  public void addAlert(HttpServletRequest req, String uid)
      throws SQLException, ParameterException, CaldataException
  {
    PEParameter pep = new PEParameter(req);
    PreparedStatement s = null;
    String description;
    int shourInt, ehourInt;
    try {
      s = getPreparedStatement(ADD_ALERT);
      String smonth = pep.getStartMonth();
      String sday = pep.getStartDay();
      String syear = pep.getStartYear();
      if (sday.length() < 2) {
        sday = "0" + sday;
      }
      String startdate = syear + "-" + monthNumber(smonth) + "-" + sday;
      s.setString(1, startdate);
      String emonth = pep.getEndMonth();
      String eday = pep.getEndDay();
      String eyear = pep.getEndYear();
      if (eday.length() < 2) {
        eday = "0" + eday;
      }
      String enddate = eyear + "-" + monthNumber(emonth) + "-" + eday;
      s.setString(2, enddate);
      s.setString(3, Fields.ESHORTDESC.sizeToFit(pep.getShortdesc()));
      description = pep.getLongdesc();
      s.setString(4, Fields.getDBLongdesc(description));
      s.setString(5, Fields.getDBLongdesc1(description));
      if (pep.getLink() != null) {
        temp_string = pep.getLink();
        temp_string = ConvertRoutines.insertHttp(temp_string);
        s.setString(6, Fields.ELINK.sizeToFit(temp_string));
      } else {
        s.setString(6, null);
      }
      s.setString(7, Fields.ECREATOR.sizeToFit(uid));
      s.executeUpdate();
    } finally {
      close();
    }
  }

  public void updateEvents(HttpServletRequest req, 
                           int locationid, int sponsorid)
      throws SQLException, ParameterException, CaldataException
  {
    PEParameter pep = new PEParameter(req);
    User user = new User(pep.getCreator());

    Event ev = new Event (pep, user,
                          locationid, sponsorid);

    try {
      ev.replace(user);
    } finally {
      // close();
    }
  }

  private static final String UPDATE_ALERTS = 
      "update " + Tables.EVENTS + " set " +
      Fields.ESEQ.getInsertName() + "= " +
      Fields.ESEQ.getInsertName() + " + 1, " +
      Fields.ELASTMOD.getInsertName() + "=?, " +
      Fields.ESTARTDATE.getInsertName() + "=?, " +
      Fields.EENDDATE.getInsertName() + "=?, " +
      Fields.ESHORTDESC.getInsertName() + "=?, " +
      Fields.ELONGDESC.getInsertName() + "=?, " +
      Fields.ELONGDESC1.getInsertName() + "=?, " +
      Fields.ELINK.getInsertName() + "=? where " +
      Fields.EEVENT_ID.getInsertName() + "=?";

  /** Update an alert
    @param req HTTP request object with data for the alert
    @exception SQLException If there's a database access problem
    @exception ParameterException If there's a problem with the form data
   */
  public void updateAlert(HttpServletRequest req) 
      throws SQLException, ParameterException
  {
    PreparedStatement s = null;
    String description;
    PEParameter pep = new PEParameter(req);

    try {
      s = getPreparedStatement(UPDATE_ALERTS);
      s.setTimestamp(1, new java.sql.Timestamp(new java.util.Date().getTime()));

      String smonth = pep.getStartMonth();
      String sday = pep.getStartDay();
      String syear = pep.getStartYear();

      if (sday.length() < 2) {
        sday = "0" + sday;
      }

      String startdate = syear + "-" + monthNumber(smonth)
                         + "-" + sday;
      s.setString(2, startdate);

      String emonth = pep.getEndMonth();
      String eday = pep.getEndDay();
      String eyear = pep.getEndYear();

      if (eday.length() < 2) {
        eday = "0" + eday;
      }

      String enddate = eyear + "-" + monthNumber(emonth)
                       + "-" + eday;
      s.setString(3, enddate);

      s.setString(4, Fields.ESHORTDESC.sizeToFit(pep.getShortdesc()));
      description = pep.getLongdesc();
      s.setString(5, Fields.getDBLongdesc(description));
      s.setString(6, Fields.getDBLongdesc1(description));

      if (pep.getLink() != null) {
        temp_string = pep.getLink();
        temp_string = ConvertRoutines.insertHttp(temp_string);
        s.setString(7, Fields.ELINK.sizeToFit(temp_string));
      } else {
        s.setString(7, null);
      }

      s.setInt(8, pep.getEventID());
      s.executeUpdate();
    } finally {
      close();
    }
  }

  /** SQL to delete an event */
  private static final String DELETE_EVENTS =
      "delete from " + Tables.EVENTS + " where " +
      Fields.EEVENT_ID.getInsertName() + "=?";

  public void deleteEvents(int eventid)
    throws SQLException
  {
      PreparedStatement s = null;

      try {
          s = getPreparedStatement(DELETE_EVENTS);
          s.setInt(1, eventid);
          s.executeUpdate();
      } finally {
          close();
      }
  }

  /** SQL to delete an entry in the authorization table */
  private static final String DELETE_AUTH =
      "delete from " + Tables.AUTH + " where " +
      Fields.AUSERID.getInsertName() + "=?";

  public void deleteAuth(int userid)
    throws SQLException
  {
      PreparedStatement s = null;

      try {
          s = getPreparedStatement(DELETE_AUTH);
          s.setInt(1, userid);
          s.executeUpdate();
      } finally {
          close();
      }
  }

  private static final String SELECT_KEYWORDS = 
    "select "
    + Fields.KKEYWORD_ID.getSelectName() + " from "
    + Tables.KEYWORDS.getName() + " where " 
    + Fields.KWORD.getStmtName() + "=?" + AND
    + Fields.KCREATOR.getStmtName() + "=?" + AND
    + Fields.KPUBLIC.getStmtName() + "=\'T\';";

  public ResultSet selectKeywords(HttpServletRequest req, User user)
    throws SQLException
  {
  PEParameter pep = new PEParameter(req);
      PreparedStatement s = null;
      ResultSet rs = null;

          s = getPreparedStatement(SELECT_KEYWORDS);
          s.setString(1, pep.getWord());
          s.setString(2, user.getName());
          rs = s.executeQuery();
      return rs;
  }

  private static final String ADD_KEYWORDS = 
      "insert into " + Tables.KEYWORDS + " (" +
      Fields.KWORD.getInsertName() + ", " +
      Fields.KCREATOR.getInsertName() + ", " +
      Fields.KPUBLIC.getInsertName() + ") values (?, ?, \'T\');";

  public void addKeywords(HttpServletRequest req, String uid)
    throws SQLException
  {
  PEParameter pep = new PEParameter(req);
      PreparedStatement s = null;

      try {
          s = getPreparedStatement(ADD_KEYWORDS);
          s.setString(1, Fields.KWORD.sizeToFit(pep.getWord()));
          s.setString(2, Fields.KCREATOR.sizeToFit(uid));
          s.executeUpdate();
      } finally {
          close();
      }
  }

  private static final String SELECT_EVENT_KEYWORDS = 
    "select " 
    + Fields.EKEVENT_ID.getSelectName() + " from "
    + Tables.EVENTKEYWORDS.getName() + " where " 
    + Fields.EKEVENT_ID.getStmtName() + "=?" + AND
    + Fields.EKKEYWORD_ID.getStmtName() + "=?;";

  public ResultSet selectEventKeywords(int eventid, int keywordid)
    throws SQLException
  {
      PreparedStatement s = null;
      ResultSet rs = null;

          s = getPreparedStatement(SELECT_EVENT_KEYWORDS);
          s.setInt(1, eventid);
          s.setInt(2, keywordid);
          rs = s.executeQuery();
      return rs;
  }

  private static final String SELECT_AUTH = 
    "select * from " + Tables.AUTH.getName() + " where "
    + Fields.AUSERID.getStmtName() + " =?"; 

  public ResultSet selectAuth(int userid)
    throws SQLException
  {
      PreparedStatement s = null;
      ResultSet rs = null;

          s = getPreparedStatement(SELECT_AUTH);
          s.setInt(1, userid);
          rs = s.executeQuery();
      return rs;
  }

  /** SQL to add an entry into the event_keywords table */
  private static final String ADD_EVENT_KEYWORDS = 
      "insert into " + Tables.EVENTKEYWORDS + " (" +
      Fields.EKEVENT_ID.getInsertName() + ", " +
      Fields.EKKEYWORD_ID.getInsertName() + ") values (?, ?);";

  public void addEventKeywords(int eventid, int keywordid)
      throws SQLException
  {
      PreparedStatement s = null;

      try {
          s = getPreparedStatement(ADD_EVENT_KEYWORDS);
          s.setInt(1, eventid);
          s.setInt(2, keywordid);
          s.executeUpdate();
      } finally {
          close();
      }
  }

  /** SQL to delete an entry in the event_keywords table */
  private static final String DELETE_EK = 
      "delete from " + Tables.EVENTKEYWORDS + " where " +
      Fields.EKEVENT_ID.getInsertName() + "=?";

  public void deleteEventKeywords(int eventid)
    throws SQLException
  {
      PreparedStatement s = null;

      try {
          s = getPreparedStatement(DELETE_EK);
          s.setInt(1, eventid);
          s.executeUpdate();
      } finally {
          close();
      }
  }

// execute sql update string
  public void executeUpdate(String update_string) throws SQLException
  {
      PreparedStatement s = null;

      try {
          s = getPreparedStatement(update_string);
          s.executeUpdate();
      } finally {
          close();
      }
  }

      // convert month name to it's corresponding month number
  public String monthNumber(String month) 
  {
    String month_num;

    if (month.startsWith("Jan")) {
      month_num = "01";
    } else if (month.startsWith("Feb")) {
      month_num = "02";
    } else if (month.startsWith("Mar")) {
      month_num = "03";
    } else if (month.startsWith("Apr")) {
      month_num = "04";
    } else if (month.startsWith("May")) {
      month_num = "05";
    } else if (month.startsWith("Jun")) {
      month_num = "06";
    } else if (month.startsWith("Jul")) {
      month_num = "07";
    } else if (month.startsWith("Aug")) {
      month_num = "08";
    } else if (month.startsWith("Sep")) {
      month_num = "09";
    } else if (month.startsWith("Oct")) {
      month_num = "10";
    } else if (month.startsWith("Nov")) {
      month_num = "11";
    } else if (month.startsWith("Dec")) {
      month_num = "12";
    } else {
      month_num = "00";
    }

    return month_num;
  }

  /** Print the queries used in the class to standard output
    @param args ignored
   */
  public static void main(String[] args)
  {
    System.out.println(SELECT_SPONSOR_FROM_EVENTS);
       // System.out.println(ADD_SPONSORS);
    System.out.println(SELECT_LOCATION_FROM_EVENTS);
       // System.out.println(ADD_LOCATIONS);
    System.out.println(ADD_AUTH);
       // System.out.println(ADD_EVENTS);
       // System.out.println(ADD_ALERT);
       // System.out.println(UPDATE_EVENTS);
       // System.out.println(UPDATE_ALERTS);
    System.out.println(DELETE_EVENTS);
    System.out.println(DELETE_AUTH);
    System.out.println(SELECT_KEYWORDS);
    System.out.println(ADD_KEYWORDS);
    System.out.println(SELECT_EVENT_KEYWORDS);
    System.out.println(SELECT_AUTH);
    System.out.println(ADD_EVENT_KEYWORDS);
    System.out.println(DELETE_EK);
  }
} 
