// java

package pubevents.util;

import java.util.Iterator;

import edu.washington.cac.calendar.data.Keyword;
import edu.washington.cac.calendar.data.NoSuchItemException;
import edu.washington.cac.calendar.data.PublicEvents;
import edu.washington.cac.calendar.db.DBResources;

import pubevents.PEParameter;

/**
  Utilities to act on data objects
  @author Greg Barnes
  @version 1.0
 */
public class DataUtil
{
  /**
    Given form inputs, find the keyword corresponding to the inputs
    @param pep form inputs
    @return the keyword corresponding to the inputs
    @exception NoSuchItemException If there is no such keyword
   */
  public static Keyword getKeyword(PEParameter pep) 
      throws NoSuchItemException
  {
    Iterator i = PublicEvents.getPublicKeywords().elements();

    while (i.hasNext()) {
      Keyword k = (Keyword) i.next();

      if (k.getWord().equals(pep.getWord()) && 
          (k.getCreator().equals(DBResources.officialKeywordCreator()) ||
           k.getCreator().equals(DBResources.unofficialKeywordCreator())))
      {
        return k;
      }
    }

    throw new NoSuchItemException("No such public keyword: " + pep.getWord());
  }

  /**
    Is a keyword official?
    @param id Keyword id
    @return is the keyword official?  Return false if there is no such keyword
   */
  public static boolean isOfficialKeyword(int id) 
  {
    try {
      return PublicEvents.getPublicKeywords().getKeyword(id).getCreator().
          equals(DBResources.officialKeywordCreator());
    } catch (NoSuchItemException e) {
      return false;
    }
  }
}
