// java

package uwcal;

import javax.servlet.http.HttpServletRequest;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.myuw.servlet.Log;
import edu.washington.cac.myuw.servlet.ParameterException;

import uwcal.out.DisplayText;

/**
  Control class that invokes actions for the application
  @author Greg Barnes
  @version 2.1
 */
public class CalActionHandler {
  /** All the actions in one array */
  private static CalAction[] actions = 
      new CalAction[CalParameter.DISPLAY_FUNCTIONS.length + 
          CalParameter.OPERATIONS.length + 
          CalParameter.DELETE_OPERATIONS.length +
          CalParameter.REF_OPERATIONS.length +
          CalParameter.FILTER_OPERATIONS.length];

  /** Initialize the actions array */
  static {
    int i;

    for (i = 0; i < CalParameter.DISPLAY_FUNCTIONS.length; i++) {
      actions[i] = new PageAction(CalParameter.DISPLAY_FUNCTIONS[i]);
    }

    for (i = 0; i < CalParameter.OPERATIONS.length; i++) {
      actions[i + CalParameter.DISPLAY_FUNCTIONS.length] = 
          new DBAction(CalParameter.OPERATIONS[i], 
                       DisplayText.OP_MESSAGES[i]);
    }

    for (i = 0; i < CalParameter.DELETE_OPERATIONS.length; i++) {
      actions[i + CalParameter.DISPLAY_FUNCTIONS.length +
              CalParameter.OPERATIONS.length] = 
          new DeleteAction(CalParameter.DELETE_OPERATIONS[i], 
                           DisplayText.DELETE_OP_MESSAGES[i]);
    }

    for (i = 0; i < CalParameter.REF_OPERATIONS.length; i++) {
      actions[i + CalParameter.DISPLAY_FUNCTIONS.length +
              CalParameter.OPERATIONS.length +
              CalParameter.DELETE_OPERATIONS.length] = 
          new RefAction(CalParameter.REF_OPERATIONS[i], 
                             DisplayText.REF_OP_MESSAGES[i]);
    }

    for (i = 0; i < CalParameter.FILTER_OPERATIONS.length; i++) {
      actions[i + CalParameter.DISPLAY_FUNCTIONS.length +
              CalParameter.OPERATIONS.length +
              CalParameter.DELETE_OPERATIONS.length +
              CalParameter.REF_OPERATIONS.length] = 
          new FilterAction(CalParameter.FILTER_OPERATIONS[i]);
    }
  }

  /**
    Create the appropriate CalAction object to handle a request
    @param req The HTTP request object
    @param user The user who is performing the action
    @param es The events for the user
    @param log A file to print messages to
   */
  public static CalAction create(HttpServletRequest req, 
                                 User user,
                                 EventsI es,
                                 Log log)
      throws ParameterException
  {
    MyCalendar cal;

    try {
      cal = new MyCalendar(new CalParameter(req).getDateString());
    } catch (CaldataException e) {
      // bogus date string; use current day
      cal = new MyCalendar();
    }

    String function = new CalParameter(req).getFunction();

    if (function == null) {
      function = CalParameter.DEFAULT_VIEW;
    }

    int i;

    for (i = 0; i < actions.length; i++) {
      if (actions[i].handles(function)) {
        return actions[i].newInstance(cal, user, es, 
                                      new CalParameter(req), log);
      }
    }

    return 
        new IllegalAction(function, cal, user, es, 
                          new CalParameter(req), log);
  }
}
