// java

package uwcal;

import java.io.IOException;
import java.sql.SQLException;

import javax.servlet.http.HttpServletResponse;

import org.apache.ecs.Entities;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.ItemException;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.calendar.filter.FilteredEvents;
import edu.washington.cac.myuw.servlet.Log;

import uwcal.filter.FilterParser;
import uwcal.out.Page;

/**
  An action in the calendar application that changes the filters for
  a guest view

  @author Greg Barnes
  @version 1.0
 */
public class FilterAction extends CalAction 
{
  /**
    Create a minimal FilterAction, not suitable for performing actions.
    This object can be used to match function names with the proper 
    FilterAction class, and calling newInstance to create a more complete 
    version of the object which can perform the action

    @param function Name of the function this FilterAction can handle
    @param successMessage message to be displayed if the action succeeds
   */
  FilterAction(String function) 
  {
    super(function);
  }

  /** Create a new FilterAction to perform actions

    @param function The function that was called
    @param cal The day for the action
    @param user The user performing the action
    @param es The EventsI object for the user
    @param cp The form parameters
    @param log A file to print messages to
   */
  protected FilterAction(String function,       
                         MyCalendar cal,
                         User user, 
                         EventsI es,
                         CalParameter cp,
                         Log log)
  {
    super(function, cal, user, es, cp, log);
  }

  /** Create a new FilterAction
    @param cal The day for the action
    @param user The user performing the action
    @param es The EventsI object for the user
    @param cp The form parameters
    @param log A file to print messages to
   */
  CalAction newInstance(MyCalendar cal, 
                        User user, 
                        EventsI es, 
                        CalParameter cp,
                        Log log) 
  {
    return new FilterAction(getHandlesFunction(), cal, user, es, cp, log);
  }

  /**
    Perform the action on the object 
   */
  protected void basePerformAction() 
  {
    if (!this.user.isGuest()) {
      return;        // non-guests don't mess with filters
    }
    
    ((FilteredEvents) this.es).setFilter(new FilterParser(cp).getFilter());
  }

  /**
    Print the page to the servlet's output stream
    @param resp The servlet's response object
    @exception IOException if there's a problem printing the page
    @exception java.sql.SQLException if there's a problem with the database
    @exception CaldataException if the database contains bad data
   */
  protected void baseGeneratePage(HttpServletResponse resp) 
    throws IOException, SQLException, CaldataException
  {
    try {
      Page.create(cp.getFunction2(), cal, this.user,
                  es, message, cp).print(resp);
    } catch (ItemException e) {
      throw new RuntimeException(
        "creating simple view shouldn't cause ItemException, but it did");
    }
  }
}
