/*++Config.java+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Log:
 * see eof
 */


package  uwcal.export;


import  java.util.GregorianCalendar;

import  edu.washington.cac.myuw.servlet.ParameterException;

import  edu.washington.cac.calendar.CalParameter;
import  edu.washington.cac.calendar.icalendar.Utilities;


/**
 * Export config values and utilities.
 *
 * @author  slh
 * @version  0.02 2003/10/13 slh
 */
public
class  Config
{

/*----------------------------------------------------------------------------
 *						Class Package Methods
 *--------------------------------------------------------------------------*/

  static
  String  errorToString  (int	Error	)
  {
    int		idx;

    for (idx = 0 ;
	 idx < Config.aCode_Error.length &&
	   idx < Config.astrLabel_Error.length ;
	 idx++) {
      if (Error == Config.aCode_Error[idx]) {
	return Config.astrLabel_Error[idx];			/*RETURN*/
      }
    }

    return errorToString( Config.Error_Unknown );		/*RETURN*/
  }


  static
  String  formatToString  (int	Format	)
  {
    int		idx;

    for (idx = 0 ;
	 idx < Config.aCode_Format.length &&
	   idx < Config.astrValue_Format.length ;
	 idx++) {
      if (Format == Config.aCode_Format[idx]) {
	return Config.astrValue_Format[idx];			/*RETURN*/
      }
    }

    /*Note: do not want something choking on null later*/
    return "";							/*RETURN*/
  }


  static
  int  stringToFormat  (String	strFormat	)
  {
    int		idx;

   /* Name */
    for (idx = 0 ;
	 idx < Config.astrValue_Format.length &&
	   idx < Config.aCode_Format.length ;
	 idx++) {
      if (strFormat.equals( Config.astrValue_Format[idx] )) {
	return Config.aCode_Format[idx];			/*RETURN*/
      }
    }
   /* Numeric (as a string) */
    try {
      return Integer.parseInt( strFormat );			/*RETURN*/
    } catch (Exception	e	) {
      return Config.Format_None;				/*RETURN*/
    }
  }


  static protected
  String  getToday  (		)
  {
    java.util.Calendar	jucalendar	= new GregorianCalendar(  );

    return jucalendar.get( jucalendar.YEAR ) +
	Config.astrValue_Month[jucalendar.get( jucalendar.MONTH )] +
	Config.astrValue_Day[jucalendar.get( jucalendar.DAY_OF_MONTH ) - 1];
  }


  static protected
  String[]  getYearList  (		)
  {
    GregorianCalendar	gc;
    int			Year;
    int			idx;
    String		astrYear[];

    gc = new GregorianCalendar(  );
    Year = gc.get( gc.YEAR );

    astrYear = new String[(Year + 5) - (Year - 5) + 1];
    for (idx = 0 ; idx < 5 + 5 + 1 ; idx++) {
      astrYear[idx] = "" + (Year - 5 + idx);
    }

    return astrYear;
  }


  static protected
  java.util.Calendar  getCheckCalendar  (String	strDate	)
  {
    int[]		aDate		= new int[3];
    GregorianCalendar	gcDate		= null;

    try {
      aDate = Utilities.splitNumber( strDate , new int[] {4, 2, 2} );
      aDate[1]--;	/* month is zero-based */

      /* test date */
      /*Note: 16th should be safe for any month*/
      gcDate = new GregorianCalendar(
			aDate[0] , aDate[1] , 16 , 0 , 0 , 0 );
      if (aDate[0] < gcDate.getActualMinimum( gcDate.YEAR ) ||
	  gcDate.getActualMaximum( gcDate.YEAR ) < aDate[0] ||
	  aDate[1] < gcDate.getActualMinimum( gcDate.MONTH ) ||
	  gcDate.getActualMaximum( gcDate.MONTH ) < aDate[1] ||
	  aDate[2] < gcDate.getActualMinimum( gcDate.DAY_OF_MONTH ) ||
	  gcDate.getActualMaximum( gcDate.DAY_OF_MONTH ) < aDate[2]) {
	gcDate = null;
      } else {
	/* real date */
	gcDate = new GregorianCalendar(
			aDate[0] , aDate[1] , aDate[2] , 0 , 0 , 0 );
      }
    } catch (NumberFormatException    e       ) {
      ;		// null statement
    }

    return gcDate;
  }


  static protected
  String  paramsToDateCode  (CalParameter	cp			,
			     String		strParamNamePrefix	)
  throws  ParameterException
  {
    try {
      /* attempt to get the value in the url */
      return cp.getPValue( strParamNamePrefix );		/*RETURN*/
    } catch (Exception	e	) {
      /* then try to get the value in the form */
      return cp.getPValue(
		strParamNamePrefix + Config.strParamNameBase_Year ) +
	cp.getPValue( strParamNamePrefix + Config.strParamNameBase_Month ) +
	cp.getPValue( strParamNamePrefix + Config.strParamNameBase_Day );
								/*RETURN*/
    }
  }


/*----------------------------------------------------------------------------
 *						Class Package Constants
 *--------------------------------------------------------------------------*/
 /* Config/Defaults */
  static final  String	strDefault_Name		= "";
  static final  int	iDefault_Format		= Config.Format_Text;
  static final  boolean	bDefault_MIME		= true;
  static final  boolean	bDefault_QP		= true;

  static final  String	strFilename_ICalendar	= "calendar.ics";
  static final  String	strFilename_MIME	= "calendar.mime";

 /* Enumerations */
  static final  int	Format_None		= -1;
  static final  int	Format_iCalendar	= 0;
  static final  int	Format_XML		= 1;
  static final  int	Format_Text		= 2;

  static final  int	Error_NoError		= 0;
  static final  int	Error_MissingData	= 1;
  static final  int	Error_InvalidData	= 2;
  static final  int	Error_NullParameter	= 3;
  static final  int	Error_Unknown		= 4;

 /* Parameters and Values */
  static final  String	strParamName_Name	= "name";
  static final  String	strParamLabel_Name	= "Calendar";
  static final  String	strParamAccess_Name	= "c";
  static final  String	strParamDescr_Name	=
		"Name of calendar from which to retrieve data.";
  static final  String	strParamName_Begin	= "begin";
  static final  String	strParamLabel_Begin	= "Begin Date";
  static final  String	strParamAccess_Begin	= "b";
  static final  String	strParamDescr_Begin	=
		"Beginning date of time period for which to retrieve data.";
  static final  String	strParamName_End	= "end";
  static final  String	strParamLabel_End	= "End Date";
  static final  String	strParamAccess_End	= "e";
  static final  String	strParamDescr_End	=
		"Ending date of time period for which to retrieve data.";
  static final  String	strParamNameBase_Month	= "_month";
  static final  String	strParamNameBase_Day	= "_day";
  static final  String	strParamNameBase_Year	= "_year";
  static final  String	strParamName_Mime	= "mime";
  static final  String	strParamLabel_Mime	= "MIME Encapsulate?";
  static final  String	strParamAccess_Mime	= "m";
  static final  String	strParamDescr_Mime	=
		"MIME encapsulate (iCalendar)?";
  static final  String	strParamName_QP		= "QP";
  static final  String	strParamLabel_QP	= "Quoted-Printable Encode?";
  static final  String	strParamAccess_QP	= "q";
  static final  String	strParamDescr_QP	=
		"quoted-printable encode (iCalendar in MIME encapsulation)?";
  static final  String	strParamName_Format	= "format";
  static final  String	strParamLabel_Format	= "Format";
  static final  String	strParamAccess_Format	= "f";
  static final  String	strParamDescr_Format	=
		"Format in which to retrieve data.";
  static final  String	strParamName_Error	= "error";
  static final  String	strParamName_Retrieve	= "retrieve";
  static final  String	strParamLabel_Retrieve	= "Retrieve";

  /*Note: corresponding label and value arrays must correspond*/
  static final  String	astrLabel_Month[]	= {
    "Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec",
  };
  static final  String	astrValue_Month[]	= {
    "01","02","03","04","05","06","07","08","09","10","11","12",
  };
  /*Note: corresponding label and value arrays must correspond*/
  static final  String	astrLabel_Day[]		= {
    "01","02","03","04","05","06","07","08","09","10",
    "11","12","13","14","15","16","17","18","19","20",
    "21","22","23","24","25","26","27","28","29","30","31",
  };
  static final  String	astrValue_Day[]		= astrLabel_Day;
  /*Note: corresponding label, value and code arrays must correspond*/
  static final  String	astrLabel_Format[]	= {"text","iCalendar","XML"};
  static final  String	astrValue_Format[]	= {"text","icalendar","xml"};
  static final  int	aCode_Format[]		= {
    Format_Text,Format_iCalendar,Format_XML,
  };
  /*Note: corresponding label and code arrays must correspond*/
  static final  String  astrLabel_Error[]	= {
    "No error; why are you seeing this?",
    "One or more fields were not filled-in.",
    "One or more fields contained invalid data.",
    "Unknown error.",
    "Unknown error.",
  };
  static final int	aCode_Error[]		= {
    Error_NoError, Error_MissingData, Error_InvalidData,
    Error_NullParameter, Error_Unknown,
  };

 /* HTTP & MIME header values */
  static final  String	strHeaderValue_ContentDisposition_ICalendar	=
		"attachment; filename=\"" + strFilename_ICalendar + "\"";
  static final  String	strHeaderValue_ContentDisposition_MIME	=
		"attachment; filename=\"" + strFilename_MIME + "\"";

 /* Standard Strings */
  static final  String  strCharset_UTF8		= "UTF-8";
  static final  String	strContentType_text	= "text/plain";
  static final  String	strContentType_html	= "text/html";
  static final  String	strContentType_calendar	= "text/calendar";
  static final  String	strContentType_xml	= "text/xml";
  static final  String	strContentType_mime	= "text/mime";
  static final  String	strContentTypeParamCharsetUTF8	=
				"charset=" + strCharset_UTF8;
  static final  String	strHeaderName_ContentDisposition	=
				"Content-Disposition";
  static final  String	strHeaderName_Pragma	= "Pragma";
  static final  String	strHeaderValue_NoCache	= "no-cache";

}


/* Note: update class header too */
/* Log:
 *  0.02  2003/10/13  slh
 *        define defaults for http params
 *        format options order
 *  0.01  2003/09/22, 2003/09/24  slh
 *        content disposition
 *        QP param
 *  0.00  2003/07/01 - 2003/07/02, 2003/07/16, 2003/07/23, 2003/09/03  slh
 *        create from PublicEntriesPage
 */
/*--Config.java-------------------------------------------------------------*/
