// java

package uwcal.out;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;

import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.User;

/** An object that prints Edit/Delete links for any CalendarObject
    @author Greg Barnes
    @version 1.1
  */
public abstract class EditDelete 
{
  protected HelperPrinter hp;
  protected MyCalendar today;
  protected User user;

  /**
    Create a new instance
    @param hp Printer of the <code>CalendarObject</code>
    @param today Current day
    @param user User of the application
   */
  EditDelete(HelperPrinter hp, MyCalendar today, User user) {
    this.hp = hp;
    this.today = today;
    this.user = user;
  }

  protected abstract Element editItem();
  protected abstract Element deleteItem();

  /** 
    Get HTML to use to remove the <code>CalendarObject</code>
    @return HTML to use to remove the <code>CalendarObject</code>
   */
  protected abstract Element removeItem();

  /** 
    Get HTML to use to remove all instances of the <code>CalendarObject</code>
    @return HTML to use to remove all instances of <code>CalendarObject</code>
   */
  protected abstract Element removeAllItem();

  /** 
    Get HTML to use when the <code>CalendarObject</code> can't be 
      edited or removed 
    @return HTML to use when the <code>CalendarObject</code> can't be 
      edited or removed 
   */
  protected abstract Element notCreator();

  protected abstract Element separator();

  protected Element initialSpacer() {
    return separator();
  }
  
  /** 
    Get HTML for the remove Link 
    @param style Style for the link
    @return HTML for the remove Link 
   */
  protected Element removeLink(String style) 
  {
    return Util.linkItem(hp.removeUrl(today, user.getIdentifier()), 
			 DisplayText.REMOVE, style);
  }

  /** 
    Get HTML for the remove Link 
    @return HTML for the remove Link 
   */
  protected Element removeLink() 
  {
    return removeLink(Style.LINK_ITEM);
  }

  /** 
    Get HTML for the remove all Link 
    @param style Style for the link
    @return HTML for the remove all Link 
   */
  protected Element removeAllLink(String style) 
  {
    return Util.linkItem(hp.removeAllUrl(today, user.getIdentifier()), 
			 DisplayText.REMOVE_ALL, style);
  }

  /** 
    Get HTML for the remove all Link 
    @return HTML for the remove all Link 
   */
  protected Element removeAllLink() 
  {
    return removeAllLink(Style.LINK_ITEM);
  }

  protected Element editLink(String style) {
    return Util.linkItem(hp.editUrl(today, user.getIdentifier()), 
                         DisplayText.EDIT, style);
  }

  protected Element editLink() {
    return editLink(Style.LINK_ITEM);
  }

  protected Element deleteLink(String style) {
    return Util.linkItem(hp.deleteUrl(today, user.getIdentifier()), 
                         DisplayText.DELETE, style); 
  }

  protected Element deleteLink() {
    return deleteLink(Style.LINK_ITEM);
  }

  /** @return HTML for the Edit/Delete element */
  public Element print() 
  {
    ElementContainer ec = new ElementContainer().addElement(initialSpacer()).
                                                 addElement("\n");

    if (hp.canBeEditedBy(this.user)) {
      return ec.addElement(editItem()).addElement("\n").
                addElement(separator()).addElement("\n").
                addElement(deleteItem()).addElement("\n");
    } else if (hp.canBeRemoved()) {
      if (hp.hasMultipleInstances()) {
        return ec.addElement(removeAllItem()).addElement("\n");
      } else {
        return ec.addElement(removeItem()).addElement("\n");
      }
    } else {
      return notCreator();
    }
  }
}
