// java

package uwcal.out;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.Event;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.ItemAccessException;
import edu.washington.cac.calendar.data.NoSuchItemException;
import edu.washington.cac.calendar.data.User;

import uwcal.out.forms.CalendarForm;
import uwcal.out.forms.EditEventForm;

/**
  Page to edit an event
  @author Greg Barnes
  @version 1.1
 */
public class EditEvent extends EditEntity 
{
  /** Create a new instance
    @param es user's event cache
    @param message Message to print at top of the page
    @param view View to return to when we're done
    @param e Event to edit.  Should contain the correct id, but need not
      contain other fields

    @exception NoSuchItemException If an event with the matching id does
      not exist
    @exception ItemAccessException If the user cannot edit the event
   */
  EditEvent(EventsI es, 
            String message, 
            Timeview view,
            Event e) 
      throws NoSuchItemException, ItemAccessException
  {
    super(es, message, view, DisplayText.EDIT_TITLE, 
          es.getEvent(e.getId()));
  }

  /**
    Get the form used by this page
    @return the form used by this page
   */
  protected CalendarForm getForm()
  {
    return new EditEventForm(this, (Event) this.c, this.es.getLocations());
  }

  /** Return the URL for editing the event.
    @param e Event to edit.
    @param mc Current date
    @param page Page this URL is a part of
    @param sid The session identifier
  */
  public static String url(Event e, MyCalendar mc, Page page, int sid) 
  {
    return EditEntity.url(CalParameter.EDITEVENT_FUNC, Util.EVENT_ID_PARAM,
                          mc, page, e, sid); 
  }
}
