// java

package uwcal.out;

import java.util.Calendar;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.Entities;
import org.apache.ecs.html.Span;
import org.apache.ecs.html.TD;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.Event;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.User;

/** An object that prints an event
    @author Greg Barnes
    @version 1.1
 */
public class HourlyEventPrinter extends SingleDayEventPrinter 
{
  /** Create a new instance
    @param user User of the app
    @param today Current day
   */
  protected HourlyEventPrinter(User user, MyCalendar today, Timeview view) 
  {
    super(user, today, view);
  }

  protected String noTimeRange() {
      return Entities.NBSP;
  }

  protected String linkStyle() {
      return null;
  }

  protected String spanStyle() {
      return null;
  }

  protected String timeStyle() {
      return null;
  }

  /** 
    Get the More/Edit/Delete links (or the equivalent) for an event
    @param e Event in question
    @return the More/Edit/Delete links (or the equivalent) for an event
   */
  protected Element moreEditDelete(Event e) 
  {
    return new TD().
        addElement(new Span().
            addElement(
                new MoreEditDelete(
                    new EventHelperPrinter(e, this.view), 
                                       this.today, this.user).
                    print()).
                addElement("\n").
                setClass(Style.AUX)).
        setClass(Style.EVENTLIST_LINKS);
  }

  /**
    Does an event start in the AM today?
    @param e The Event to test
    @return whether it's an AM event
   */
  protected boolean isAm(Event e) {
    return e.startCalendar().hasTime() && 
           e.startCalendar().isSameDay(this.today) && 
           e.startCalendar().get(Calendar.HOUR_OF_DAY) < 12;
  }
    
  /**
    Does an event start in the PM today?
    @param e The Event to test
    @return whether it's a PM event
   */
  protected boolean isPm(Event e) {
    return e.startCalendar().hasTime() && 
           e.startCalendar().isSameDay(this.today) && 
           e.startCalendar().get(Calendar.HOUR_OF_DAY) > 11;
  }
    
  /**
    Return the style for the time displayed for an event
    @param e The Event in question
    @return The style for the cell that displays the time 
   */
  private String timeclass(Event e) 
  {
    if (isPm(e)) {
      return Style.EVENTLIST_PMTIME;
    } else if (isAm(e)) {
      return Style.EVENTLIST_AMTIME;
    } else {
      return(Style.EVENTLIST_NOTIME);
    }
  }

  /**
    Get HTML for the formatted event
    @param e The event
    @param es The user's cache of events
    @return the formatted event
   */
  protected Element format(Event e, EventsI es) 
  {
    return new ElementContainer().
           addElement(new TD().addElement(timeRange(e)).setClass(timeclass(e))).
           addElement("\n").
           addElement(new TD().
                      addElement(addIcon(e)).
                      addElement(eventDesc(e)).
                      addElement(location(e, es)).
                      setClass(Style.EVENTLIST_DESC)).
           addElement("\n").
           addElement(moreEditDelete(e));
  }
}
