// java

package uwcal.out;

import java.util.Calendar;
import java.util.Date;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.html.BR;
import org.apache.ecs.html.Table;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TR;

import edu.washington.cac.myuw.servlet.session.UWSession;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.User;

/**
  A page that shows a year
  @author Greg Barnes
  @version 1.1
 */
public class Yearview extends Timeview
{
  /** The function name for a year view */
  private static final String FUNC = CalParameter.YEARVIEW_FUNC;

  /** Create a new instance
    @param mc Current day
    @param user User of the app
    @param es User's events cache
    @param message Message to print at the top of the page
   */
  protected Yearview(MyCalendar mc, User user, EventsI es, String message) 
  {
    super(mc, user, es, message, FUNC,
          new MyCalendar(mc.firstDayOfThisYear()),
          new MyCalendar(mc.lastDayOfThisYear()));
  }

  /**
    @return A url that will yield a page to edit something
    @param mc Current date for the page
   */
  public static String url(MyCalendar mc) 
  {
    return url(mc, null);
  }

  /**
    @return A url that will yield a page to edit something
    @param mc Current date for the page
    @param prefix URL prefix to use instead of the default
   */
  public static String url(MyCalendar mc, String prefix) 
  {
    return (baseUrl(FUNC, prefix) + mc.getDateDigits());
  }

  /**
    Get the object that can print the prev/this/next links
    @return the object that can print the prev/this/next links
   */
  protected DateHeader getDateHeader() 
  {
    return new YearDateHeader(this.mc);
  }

  /**
    Is this a year view?
    @return true
   */
  protected boolean isYearview() 
  {
    return true;
  }

  private static final int MONTHS_IN_ROW = 4;
  static final String MONTH_CELL_WIDTH = (100 / MONTHS_IN_ROW) + "%";

  /** @return The table cell for a particular month
    @param month The month in question
   */
  private TD monthCell(MyCalendar month) 
  {
    TD monthCell = new TD().
setWidth(MONTH_CELL_WIDTH).                //%%% for non-compliant browsers
        addElement(new SmallMonthPrinter(month, es).print());

    if (month.get(Calendar.MONTH) == mc.get(Calendar.MONTH)) {
      monthCell.setID(Style.CURRENT_MONTH);
    } else if (month.get(Calendar.MONTH) == 
        new MyCalendar().get(Calendar.MONTH) && 
        month.get(Calendar.YEAR) == new MyCalendar().get(Calendar.YEAR))
    {
      monthCell.setID(Style.THIS_MONTH);
    } else {
      monthCell.setClass(Style.MONTH_CELL);
    }

    return monthCell;
  }

    private Element yearGrid() {
        ElementContainer ec = new ElementContainer();
        TR tr = new TR().addElement("\n");

        int i;
        MyCalendar currentMonth = new MyCalendar(mc.firstMonthOfThisYear());

        for (i = 0; i < mc.monthsInYear(); 
          i++, currentMonth = currentMonth.addTime(Calendar.MONTH, 1))
        {
            tr.addElement(monthCell(currentMonth));

            if (i % MONTHS_IN_ROW == MONTHS_IN_ROW - 1) {
                ec.addElement(tr).addElement("\n");
                tr = new TR().addElement("\n");
            }
        }

        if (mc.monthsInYear() % MONTHS_IN_ROW != 3) {
            ec.addElement(tr).addElement("\n");
        }

        return ec;
    }

    protected Element contents() 
    {
        return new ElementContainer().addElement(new BR()).addElement("\n").
          addElement(new Table().setCellSpacing(2).setCellPadding(3).
setWidth("100%").                //%%% for non-compliant browsers
            addElement(yearGrid()).addElement("\n").setID(Style.YEAR_TABLE));
    }

    protected String title() {
        return DisplayText.YEARVIEW_TITLE;
    }

  /**
    Should this page show the icon legend for guests?
    @return Should this page show the icon legend for guests?
   */
  protected boolean shouldShowIconLegend()
  {
    return false;
  }
}
