// java

package uwcal.out.forms;

import org.apache.ecs.Element;
import org.apache.ecs.StringElement;
import org.apache.ecs.html.Input;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.Keyword;
import edu.washington.cac.myuw.util.Coder;

import uwcal.out.Page;
import uwcal.out.Style;
import uwcal.out.Util;

/**
  A form to add or edit keywords
  @author Greg Barnes
  @version 1.2
 */
public abstract class KeywordForm extends CalendarForm {
  /** The old version of the key word, if we're editing it */
  protected Keyword k;

  /** Something to generate common calendar form elements */
  private CommonFormElements cfe;

  /* Create a Keyword form
     @param page Page this form is a part of
     @param k An old keyword, if we're editing a keyword, null if we're
       adding a new one
   */
  protected KeywordForm(Page page, Keyword k) 
  {
    super(page);
    this.k = k;
    cfe = new CommonFormElements(k);
  }

  /**
    Get the input tag for the main field of a keyword
    @param initialValue The initial value (for editing)
    @return the input tag
   */
  private Element wordInput(String initialValue) {
    return new Input(Input.TEXT, CalParameter.WORD, initialValue).setSize(25);
  }

  /**
    Get the HTML to input the main field of a keyword
    @return the HTML
   */
  private Element word() {
    if (k == null) {
      return cfe.newInput(Util.spanItem(FormsDisplayText.WORD, Style.STD_TEXT), 
                          wordInput(""));
    } else {
      return cfe.oldNewInput(FormsDisplayText.EDIT_OLD_WORD,
                             new StringElement(Coder.HTMLencode(k.getWord())),
                             FormsDisplayText.EDIT_NEW_WORD, 
                             wordInput(Coder.HTMLencode(k.getWord())));
    }
  }

  /**
    Get the HTML pieces that make up a keyword form
    @return An array of the HTML pieces
   */
  protected Element[] inputElements() {
    Element[] e = { word(), cfe.longdesc(), cfe.publicPrivate() };
    return e;
  }
}
