// java

package uwcal.out.forms;

import org.apache.ecs.Element;
import org.apache.ecs.StringElement;
import org.apache.ecs.html.Input;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.Sponsor;
import edu.washington.cac.myuw.util.Coder;

import uwcal.out.Page;
import uwcal.out.Style;
import uwcal.out.Util;

/**
  A form to change a sponsor
  @author Greg Barnes
  @version 1.1
 */
public abstract class SponsorForm extends CalendarForm 
{
  Sponsor s;
  CommonFormElements cfe;

  /** Create a new instance
     @param page Page this form is a part of
     @param s The sponsor. if it is null, adding new sponsor;
     otherwise, editing old sponsor
  */
  SponsorForm(Page page, Sponsor s) 
  {
    super(page);
    this.s = s;
    cfe = new CommonFormElements(s);
  }

  private Element nameInput(String initialValue) {
      return new Input(Input.TEXT, CalParameter.SPONSOR_NAME, initialValue).
        setSize(25);
  }

  private Element name() {
      if (s == null) {
          return cfe.newInput(
              cfe.requiredFieldName(FormsDisplayText.SPONSOR_NAME), 
            nameInput(""));
      } else {
          return cfe.oldNewInput(FormsDisplayText.EDIT_OLD_NAME,
            new StringElement(Coder.HTMLencode(s.getName())),
            cfe.requiredFieldName(FormsDisplayText.EDIT_NEW_NAME), 
            nameInput(Coder.HTMLencode(s.getName())));
      }
  }

  private Element phoneInput(String initialValue) {
      return new Input(Input.TEXT, CalParameter.PHONE, initialValue).
        setSize(25);
  }

  /**
    Get the phone number of a sponsor
    @return the phone number of a sponsor
   */
  private Element phone() 
  {
    if (s == null) {
      return cfe.newInput(
          Util.spanItem(FormsDisplayText.PHONE, Style.STD_TEXT), 
                        phoneInput(""));
    } else {
      return cfe.oldNewInput(FormsDisplayText.EDIT_OLD_PHONE,
          new StringElement(Coder.HTMLencode(s.getPhone())),
          FormsDisplayText.EDIT_NEW_PHONE, 
          phoneInput(Coder.HTMLencode(s.getPhone())));
      }
  }

  private Element emailInput(String initialValue) {
      return new Input(Input.TEXT, CalParameter.EMAIL, initialValue).
        setSize(25);
  }

  /**
    Get the email section of the form
    @return the email section of the form
   */
  private Element email() 
  {
    if (s == null) {
      return cfe.newInput(
          Util.spanItem(FormsDisplayText.EMAIL, Style.STD_TEXT), 
          emailInput(""));
    } else {
      return cfe.oldNewInput(FormsDisplayText.EDIT_OLD_EMAIL,
          new StringElement(Coder.HTMLencode(s.getEmail())),
          FormsDisplayText.EDIT_NEW_EMAIL, 
          emailInput(Coder.HTMLencode(s.getEmail())));
    }
  }

  protected Element[] inputElements() {
      Element[] e = { 
          name(), phone(), email(), cfe.link(FormsDisplayText.SPONSOR_LINK), 
          cfe.publicPrivate() 
        };

      return e;
  }
}
